#include "fft.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

/*
   This computes an in-place complex-to-complex FFT 
   x and y are the real and imaginary arrays of 2^m points.
   dir =  1 gives forward transform
   dir = -1 gives reverse transform 
*/
int FFT(int dir,int length,float **ox,float **oy)
{
   int n,i,j,m,L,istep;
   float c1,c2,t1,t2,theta;
   float *x,*y;

   /* Calculate the number of points */
   n = 1;
   while(n<length) n *= 2;

   if( length != n )
   {
	x = (float *)malloc(sizeof(float)*n);
	y = (float *)malloc(sizeof(float)*n);
	for (i=0;i<length;i++) 
	{
	    x[i] = (*ox)[i];	
	    y[i] = (*oy)[i];	
	}
	for (;i<n;i++) 
	{
	    x[i] = 0.;
	    y[i] = 0.;
	}
	free( (*ox) );
	free( (*oy) );
	(*ox) = x;
	(*oy) = y;
   }
   else 
   {
	x = *ox;
	y = *oy;
   }
   x -= 1;
   y -= 1;

   j = 1;
   for(i=1;i<=n;i++)
   {
	if(i<=j)
	{
	   t1 = x[j];
	   x[j] = x[i];
	   x[i] = t1;
	   t1 = y[j];
	   y[j] = y[i];
	   y[i] = t1;
	}
	m = n/2;
	do {
	   if( j <= m ) break;
	   j -= m;
	   m /= 2;
	}while( m >= 1 );
	j += m;
   }
   L = 1;
   do
   {
	istep = 2*L;
	for(m=1; m<=L; m++)
	{
	   theta = M_PI*(m-1.)*dir/L;
	   c1 = cos( theta );
	   c2 = sin( theta );
	   for( i=m; i<=n; i += istep)
	   {
		j = i+L;

		t1 = x[j]*c1 - y[j]*c2;
		t2 = x[j]*c2 + y[j]*c1;

		x[j] = x[i] - t1;
		y[j] = y[i] - t2;
		x[i] = x[i] + t1;
		y[i] = y[i] + t2;
	   }
	}
	L = istep;
   }while( L < n );

   /* Scaling for forward transform */

   if (dir == 1) 
   {
	for (i=1;i<=n;i++) 
	{
	   x[i] /= sqrt(n);
	   y[i] /= sqrt(n);
	}
   }

   return n;
}

void Folding(float *x, float *y, int N)
{
   register int i,n2;
   float temp;
   n2 = N/2;
   for(i=0;i<n2;i++)
   {
	temp = x[i];
	x[i] = x[i+n2];
	x[i+n2] = temp;
	temp = y[i];
	y[i] = y[i+n2];
	y[i+n2] = temp;
   }
}

