#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <libmseed.h>

int reclen  = -1;    /* Record length, -1 = autodetected */
int verbose =  0;    /* Verbosity level */

double timetol      = -1.0; /* Time tolerance for continuous traces */
double sampratetol  = -1.0; /* Sample rate tolerance for continuous traces */

int npts,N,wind,incr,stype,rtype,gain,doy;
double dt,df,duration,tinc,hr,lf,hf,freq,cnst,outdf;
char   *pzfile;

int main(int argc, char *argv[])
{
    MSTraceGroup *mstg = 0;
    MSTrace *mst;
    MSRecord *msr = 0;

    int retcode,i;
    int totalrecs = 0;
    int totalsamps = 0;
    FILE *fpms;


    if (argc < 8) {
        fprintf(stderr, "Unable to execute, not enough input parameters supplied (expected 5, received %d)\n", 
                            argc-1);
        fprintf(stderr, "%s mseed window_min incr_min low_freq high_freq df  [sens_logger] [constant]\n", argv[0]);
        fprintf(stderr, "	sens_logger	1 : STS2 + Q4120,Q330,  low-gain(1500v/m/s)\n");
        fprintf(stderr, "			2 : STS2 + Q4120,Q330,  high-gain(20000v/m/s)\n");
        fprintf(stderr, "			3 : STS2 + Constants\n");
        fprintf(stderr, "			4 : STS1 + Constants\n");
        fprintf(stderr, "			5 : CMG3TB + Q4120,Q330,low-gain(1500v/m/s)\n");
        fprintf(stderr, "			6 : CMG3TB + Q4120,Q330 high-gain(20000v/m/s)\n");
        fprintf(stderr, "			7 : CMG40T + Q4120,Q330 low-gain(1500v/m/s)\n");
        fprintf(stderr, "			8 : pz file\n");

	exit(2);
    }

    if((fpms = fopen(argv[1], "rb")) == NULL) 
    {
	fprintf(stderr, "Unable to open data file %s", argv[1]);
	exit(2);
    }

    wind = atoi(argv[2]);
    incr = atoi(argv[3]);
    sscanf(argv[4],"%lf",&lf);
    sscanf(argv[5],"%lf",&hf);
    sscanf(argv[6],"%lf",&outdf);
//  default: STS2 + Q4120,Q330,  low-gain(1500v/m/s)
    if( argc == 7 )  i = 1;
    else 	i = atoi(argv[7]);

    if( i < 1 || i > 8 ) i=1;

    cnst = 0.;
    switch(i)
    {
	case 1:
		stype=1;
		rtype=1;
		gain=1;
		break;
	case 2:
		stype=1;
		rtype=1;
		gain=2;
		break;
	case 3:
		stype=1;
		rtype=1;
		gain=2;
		sscanf(argv[8],"%lf",&cnst);
		break;
	case 4:
		stype=5;
		rtype=1;
		gain=2;
		sscanf(argv[8],"%lf",&cnst);
		break;
	case 5:
		stype=2;
		rtype=1;
		gain=1;
		break;
	case 6:
		stype=2;
		rtype=1;
		gain=2;
		break;
	case 7:
		stype=3;
		rtype=1;
		gain=1;
		break;
	case 8:
		stype=0;
		rtype=0;
		gain=0;
		pzfile = (char *)malloc(BUFSIZ*sizeof(char));
		strcpy(pzfile,argv[8]);
		break;
    } 




    /* Init MSTraceGroup */
    mstg = mst_initgroup (mstg);

    while ( (retcode = ms_readmsr(&msr, argv[1], reclen, NULL, NULL,
                                    1, 1, verbose-1)) == MS_NOERROR )
    {
          if ( verbose > 1)
            msr_print (msr, verbose - 2);

          mst_addmsrtogroup (mstg, msr, 1, timetol, sampratetol);

          totalrecs++;
          totalsamps += msr->samplecnt;
    }

    if ( retcode != MS_ENDOFFILE )
        fprintf (stderr, "Error reading %s: %s\n", argv[1], ms_errorstr(retcode));

      /* Make sure everything is cleaned up */
    ms_readmsr (&msr, NULL, 0, NULL, NULL, 0, 0, 0);

    mst = mstg->traces;
    while( mst )
    {
	spgram(mst);
	mst = mst->next;
    }

    return 0;
}
