#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#define PZ     0
#define STS2   1
#define CMG3TB 2
#define CMG40T 3
#define SS1    4
#define STS1   5

#define Q4128  1
#define Q330   2
#define Q730   3

#define LOWGAIN  1
#define HIGHGAIN 2


struct base {
	double real;
	double imag;
};
typedef struct base dcomplex;

dcomplex add(dcomplex f, dcomplex g);
dcomplex min(dcomplex f, dcomplex g);
dcomplex mul(dcomplex f, dcomplex g);
dcomplex did(dcomplex f, dcomplex g);
double   amp(dcomplex f);
void str2upper(char *str);
void get_cnst(FILE *fp,char *str,double *cnst);
void get_pz(FILE *fp,char *str,int *N,dcomplex **values);
void read_pz(char *pzfile,int *nz,int *np,dcomplex **zeros,dcomplex **poles,double *cnst);


float *get_inst_resp(int stype, int rtype,float df,int gain,double cnst,int N,char *pzfile)
{
    int i,j,np,nz;
    double freq,pi2,A0,rsens,ssens;
    dcomplex pvalue,zvalue,resp,freqv;
    dcomplex *poles,*zeros;
    float *aresp;

    pi2 = 2.*M_PI;


    /* CMG-3TB  */
    switch( stype )
    {
 	case CMG3TB:
	    nz = 3;
	    np = 4;

	    zeros = (dcomplex *)malloc(np*sizeof(dcomplex));
	    poles = (dcomplex *)malloc(np*sizeof(dcomplex));

	    zeros[0].real = 0.;  zeros[0].imag = 0.;
	    zeros[1].real = 0.;  zeros[1].imag = 0.;
	    zeros[2].real = 146.5;  zeros[2].imag = 0.;

	    poles[0].real = -5.89e-3;  poles[0].imag = +5.89e-3;
	    poles[1].real = -5.89e-3;  poles[1].imag = -5.89e-3;
	    poles[2].real = -7.32e+1;  poles[2].imag = +3.76e+1;
	    poles[3].real = -7.32e+1;  poles[3].imag = -3.76e+1;

	    for(i=0;i<nz;i++)
	    {
		zeros[i].real *= pi2;
		zeros[i].imag *= pi2;
	    }
	    for(i=0;i<np;i++)
	    {
		poles[i].real *= pi2;
		poles[i].imag *= pi2;
	    }
	    break;
 	case CMG40T:
	    nz = 2;
	    np = 6;

	    zeros = (dcomplex *)malloc(np*sizeof(dcomplex));
	    poles = (dcomplex *)malloc(np*sizeof(dcomplex));

	    zeros[0].real = 0.;  zeros[0].imag = 0.;
	    zeros[1].real = 0.;  zeros[1].imag = 0.;

	    poles[0].real = -3.50e+2;  poles[0].imag = +0.00e+0;
	    poles[1].real = -7.50e+1;  poles[1].imag = +0.00e+0;
	    poles[2].real = -7.07e-1;  poles[2].imag = +7.07e-1;
	    poles[3].real = -7.07e-1;  poles[3].imag = -7.07e-1;
	    poles[4].real = -6.23816e+1;  poles[4].imag = +1.35392e+2;
	    poles[5].real = -6.23816e+1;  poles[5].imag = -1.35392e+2;

	    for(i=0;i<nz;i++)
	    {
		zeros[i].real *= pi2;
		zeros[i].imag *= pi2;
	    }
	    for(i=0;i<np;i++)
	    {
		poles[i].real *= pi2;
		poles[i].imag *= pi2;
	    }
	    break;

 	case SS1:
	    nz = 2;
	    np = 2;

	    zeros = (dcomplex *)malloc(nz*sizeof(dcomplex));
	    poles = (dcomplex *)malloc(np*sizeof(dcomplex));

	    zeros[0].real = 0.;  zeros[0].imag = 0.;
	    zeros[1].real = 0.;  zeros[1].imag = 0.;

	    poles[0].real = -4.44e+0;  poles[0].imag = +4.44e+0;
	    poles[1].real = -4.44e+0;  poles[1].imag = -4.44e+0;
	    break;

 	case STS2:
	    nz = 2;
	    np = 5;

	    zeros = (dcomplex *)malloc(nz*sizeof(dcomplex));
	    poles = (dcomplex *)malloc(np*sizeof(dcomplex));

	    zeros[0].real = 0.;  zeros[0].imag = 0.;
	    zeros[1].real = 0.;  zeros[1].imag = 0.;

	    poles[0].real = -0.03701;  poles[0].imag = +0.03701;
	    poles[1].real = -0.03701;  poles[1].imag = -0.03701;
	    poles[2].real = -251.300;  poles[2].imag = +0.00000;
	    poles[3].real = -131.000;  poles[3].imag = +467.300;
	    poles[4].real = -131.000;  poles[4].imag = -467.300;
	    break;

 	case STS1:
	    nz = 2;
	    np = 4;

	    zeros = (dcomplex *)malloc(nz*sizeof(dcomplex));
	    poles = (dcomplex *)malloc(np*sizeof(dcomplex));

	    zeros[0].real = 0.;  zeros[0].imag = 0.;
	    zeros[1].real = 0.;  zeros[1].imag = 0.;

	    poles[0].real = -0.0123;  poles[0].imag = +0.0123;
	    poles[1].real = -0.0123;  poles[1].imag = -0.0123;
	    poles[2].real = -39.180;  poles[2].imag = +49.120;
	    poles[3].real = -39.180;  poles[3].imag = -49.120;
	    break;

	case PZ:
	    read_pz(pzfile,&nz,&np,&zeros,&poles,&cnst);
	    break;
    }

    switch( rtype )
    {
 	case Q4128:
 	case Q330:
	    rsens = 419430.;
	    break;
 	case Q730:
	    rsens = 1./1.9e-9;
	    break;
    }

    switch( gain )
    {
	case LOWGAIN:
	    ssens = 1500.;
	    break;
	case HIGHGAIN:
	    ssens = 20000.;
	    break;
    }

    aresp = (float *)malloc(N*sizeof(float));

    for(freq=df,j=0;j<N;j++,freq+=df)
    {
	zvalue.real = 1.;   zvalue.imag = 0.;
	pvalue.real = 1.;   pvalue.imag = 0.;
	freqv.imag = freq*pi2;
	freqv.real = 0.;
	for(i=0;i<nz;i++) zvalue = mul(zvalue,add(freqv,zeros[i]));
	for(i=0;i<np;i++) pvalue = mul(pvalue,add(freqv,poles[i]));
	resp = did(zvalue,pvalue);
	aresp[j] = (float)amp(resp);
	//printf("%6.3lf %9.4e %9.4e\n",amp(freqv),amp(resp),1./amp(resp));
	if( freq > -0.01 && freq < 0.01 ) A0 = 1./amp(resp);
	/*
	if( freq >= 0.9 && freq <= 1.0 ) A0 = 1./amp(resp);
	*/

    }

    /*
    if( debug )
    {
	fprintf(stderr,"ZEROS  %d\n",nz);
	for(i=0;i<nz;i++)
	    fprintf(stderr,"%10.4g  %10.4g\n",zeros[i].real,zeros[i].imag);
	fprintf(stderr,"POLES  %d\n",np);
	for(i=0;i<np;i++)
	    fprintf(stderr,"%10.4g  %10.4g\n",poles[i].real,poles[i].imag);
    // 20000: Velocity output of Sensor (V/m/s), 
    // 4.19430E+05: A/D conversion factor, Least significant bit (count/V) 
	if( cnst != 0. )     
	{
	    fprintf(stderr,"CONSTANT %g\n",cnst);
	}
	else
	{
	    fprintf(stderr,"CONSTANT %g\n",A0*rsens*ssens);
	    fprintf(stderr,"A0 %g\n",A0);
	    fprintf(stderr,"Sensor Sensitivity %g\n",ssens);
	    fprintf(stderr,"Logger Sensitivity %g\n",rsens);
	}
    }
    */
    if( cnst == 0. ) cnst = A0*rsens*ssens;

    for(j=0;j<N;j++) aresp[j] *= cnst;

    return aresp;
}


dcomplex add(dcomplex f, dcomplex g)
{
    dcomplex temp;

    temp.real = f.real+g.real;
    temp.imag = f.imag+g.imag;
	      
    return temp;
}
dcomplex min(dcomplex f, dcomplex g) 
{
    dcomplex m;
    m.real = f.real - g.real;
    m.imag = f.imag - g.imag;
    return m;
}
dcomplex mul(dcomplex f, dcomplex g) 
{
    dcomplex sum;
    sum.real = f.real * g.real - f.imag * g.imag;
    sum.imag = f.real * g.imag + f.imag * g.real;
    return sum;
}

dcomplex did(dcomplex f,dcomplex g) 
{
    dcomplex d;
    d.real = (f.real*g.real+f.imag*g.imag)/(g.real*g.real+g.imag*g.imag);
    d.imag = (f.real*g.imag-f.imag*g.real)/(g.real*g.real+g.imag*g.imag);
    return d;
}


double amp(dcomplex f)
{
    return sqrt(f.real*f.real + f.imag*f.imag);
}

void read_pz(char *pzfile,int *nz,int *np,dcomplex **zeros,dcomplex **poles,double *cnst)
{
    FILE *fp;

    if( (fp = fopen(pzfile,"r")) == NULL )
    {
	fprintf(stderr,"PZ file (%s) open error!\n",pzfile);
	return;
    }

    get_pz(fp,"ZEROS",nz,zeros);
    get_pz(fp,"POLES",np,poles);
    get_cnst(fp,"CONSTANT",cnst);

    fclose(fp);
}


void str2upper(char *str)
{
    while( *str )
    {
	*str = toupper(*str);
	str++;
    }
}
void get_cnst(FILE *fp,char *str,double *cnst)
{
    double v;
    char *tbuf,*ident;

    tbuf = (char *)malloc(BUFSIZ*sizeof(char));
    ident = (char *)malloc(BUFSIZ*sizeof(char));
    
    v = 0.;
    fseek(fp,0L,SEEK_SET);
    while(fgets(tbuf,BUFSIZ,fp))
    {
	sscanf(tbuf,"%s",ident);
	str2upper(ident);
	if( !strcmp(ident,str) )
	{
	   sscanf(tbuf,"%s %lf",ident,&v);
	   break;
	}
    }
    *cnst = v;

    free(tbuf);
    free(ident);
}

void get_pz(FILE *fp,char *str,int *N,dcomplex **values)
{
    int i;
    double v1,v2;
    char *tbuf,*ident;

    tbuf = (char *)malloc(BUFSIZ*sizeof(char));
    ident = (char *)malloc(BUFSIZ*sizeof(char));
    
    *N = -1;
    fseek(fp,0L,SEEK_SET);
    while(fgets(tbuf,BUFSIZ,fp))
    {
	sscanf(tbuf,"%s",ident);
	str2upper(ident);
	if( !strcmp(ident,str) )
	{
	   sscanf(tbuf,"%s %d",ident,N);
	   break;
	}
    }
    if( *N < 0 ) 
    {
	fprintf(stderr,"%s not defined!\n",str);
	return;
    }
    if( *N > 0 )
    {
	*values = (dcomplex *)malloc(*N*sizeof(dcomplex));
    }

    for(i=0;i<*N;i++)
    {
	fgets(tbuf,BUFSIZ,fp);
	sscanf(tbuf,"%s",ident);
        if( isalpha(ident[0]) )
        {
	    for(;i<*N;i++) (*values)[i].real = (*values)[i].imag = 0.;
	    break;
	}
	sscanf(tbuf,"%lf %lf",&v1,&v2);
	(*values)[i].real = v1;
	(*values)[i].imag = v2;
    }

    free(tbuf);
    free(ident);
}
