#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <libmseed.h>

extern int npts,N,wind,incr,stype,rtype,gain,doy;
extern double dt,df,duration,tinc,hr,lf,hf,freq,cnst,outdf;
extern char   *pzfile;
float *get_inst_resp(int stype, int rtype,float df,int gain,double cnst,int N,char *pzfile);
int spectrum(double *msd,int npts,float **spect);
void deMandS(double *fdata, int numSamps, double *cval);

void spgram(MSTrace *mst)
{
    int		*intData,itinc,i,j,igap,skip,nlength,N;
    float	*resp;
    double      outfreq,outlogdf,sum,t1,z0,logfreq,*dval,*cval;
    int         nsum,samplesize,ival,ndata;
    void        *sptr;
    float       fval,*spect;

    BTime itime;

    ms_hptime2btime(mst->starttime,&itime);

    dt = 1./mst->samprate;

    df = 0.;
    nlength=wind;             // length in minutes
    duration = 60.*nlength;  // window length in seconds
    
    tinc = 60.*incr;
    itinc = (int)(tinc*mst->samprate);
    npts  = (int)(duration/dt);

    t1 = (double)itime.hour*60.*60.+
	(double)itime.min*60.+
	(double)itime.sec + (double)itime.fract/10000.;

    doy = itime.day;
    hr = (double)itime.hour+(double)itime.min/60.;
    hr += 0.5*wind/60.; // middle of window

    if ( (samplesize = ms_samplesize(mst->sampletype)) == 0 )
    {
	fprintf (stderr, "Unrecognized sample type: %c\n", mst->sampletype);
    }
    dval = (double *)malloc(mst->numsamples*sizeof(double));
    ndata = mst->numsamples;
    cval = (double *)malloc(npts*sizeof(double));

    for(i=0;i<ndata;i++)
    {
	sptr = (char*)mst->datasamples + (i * samplesize);
	if( mst->sampletype == 'i' )
	{
	    ival = *(int32_t *)sptr;
	    z0 = (double)ival;
	}
	else if( mst->sampletype == 'f' )
	{
	    fval = *(int32_t *)sptr;
	    z0 = (double)fval;
	}
	else if( mst->sampletype == 'd' )
	{
	    z0 = *(double *)sptr;
	}
	dval[i] = z0;
    }
    fprintf(stderr,"dt: %lf, npts: %d, lf: %lf, hf: %lf\n",dt,ndata,lf,hf);

//    deMandS(dval, ndata);

//    if( debug )
//    {
//	fprintf(stderr,"dt: %lf\n",dt);
//	fprintf(stderr,"tinc: %lf sec, itinc:%d\n",tinc,itinc);
//	fprintf(stderr,"hr:%f\n",hr);
//    }

    for(i=0;i+npts<ndata;i+=itinc,hr+=tinc/3600.)
    {
//	if( debug )
//	    fprintf(stderr,"HR: %7.4f, %10d/%10d\n",hr,i,npts);
	if( hr > 24. ) hr -= 24.,doy++;
		
        deMandS(dval+i, npts, cval);
	N=spectrum(cval,npts,&spect);
        
	if( i == 0 )
	{
	    df = 1./(dt*N);
	    resp = get_inst_resp(stype,rtype,1./(dt*N),gain,cnst,N/2+1,pzfile);
	    for(j=1;j<=N/2;j++) resp[j-1] *= resp[j-1];
	    fprintf(stderr,"dt: %lf, N: %d, df: %lf\n",dt,N,df);
	}
		

	for(j=1,freq=df;j<=N/2;j++,freq+=df) 
	{
		spect[j] /= resp[j-1]/freq/freq/4/M_PI/M_PI;   // differentiation
//		spect[j] *= pow(2.*M_PI*freq,2.);
	}


	smooth(spect+1,N/2,5);

//	    if( debug ) fprintf(stderr,"%10d, N:%d\n",i/20,N);
	outfreq = lf;
	logfreq = log10(lf);
	outlogdf = log10(hf/lf)/outdf;
	sum = 0.;
	nsum = 0;
	for(j=1,freq=df;j<=N/2;j++,freq+=df)
	{
		if( freq >= lf && freq <= hf )
		{
		   if( freq > outfreq + df )
		   {
			fprintf(stdout,"%7.4f %8.5f %8.3f\n",
			hr/24.+doy,outfreq,10.*log10(sum/nsum));
			sum = 0.;
			nsum = 0;
			if(outfreq < 1. )
			{
			   logfreq += outlogdf;
			   outfreq = pow(10.,logfreq);
			}
			else
			   outfreq += 0.1;
		   }
		   else
		   {
			sum += spect[j];
			nsum ++;
		   }
		}
	}
	fprintf(stdout,">\n");
	free(spect);
    }
}

void deMandS(double *fdata, int numSamps, double *cval)
{
   int i;
   double flNumSamps;
   double alpha, avg, slope;

   flNumSamps = (double) numSamps;
   avg = 0;
   slope = 0;

   for(i=0;i<numSamps;i++)
   {
	avg += *(fdata+i);
	slope += *(fdata+i) * (double) i;
   }
   avg /= numSamps;
   slope = 12.0*slope/(flNumSamps*(flNumSamps*flNumSamps-1.0)) - 6.0*avg/(flNumSamps-1.0);
      
   alpha = avg - 0.5*(flNumSamps+1.0)*slope;
   for(i=0;i<numSamps;i++)
   {
	cval[i] = *(fdata+i) - (double) (i * slope + alpha);
   }
}

