#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include <libmseed.h>

extern int ntrig;

void stalta(double val0,double val1,double r1,double sta1,double lta1,
	double thp,double tk,double tsta,double tlta,
	double dt,double *r0,double *sta0,double *lta0)
{
    double alpsta,alplta,dr0,cf0;
    if( fabs(thp) < 0.0001 || fabs(tsta) < 0.0001 || fabs(tlta) < 0.00001 )
    {
	fprintf(stderr,"some filter params error\n");
	exit(-1);
    }

    if( fabs(tk) < 0.0001 )
    {
	fprintf(stderr,"stalta: thp close to zero\n");
	exit(-1);
    }

    alpsta = (tsta-dt)/tsta;
    alplta = (tlta-dt)/tlta;

    dr0 = val0 - val1;
    *r0 = (thp*r1) + dr0;
    cf0 = (*r0)*(*r0) + tk *(dr0*dr0);
    *sta0 = alpsta * sta1 + (1.-alpsta)*cf0;
    *lta0 = alplta * lta1 + (1.-alplta)*cf0;
}

void cnvt_time(double t1,int *trh,int *trm,int *trs,double *trss)
{
    int i;
    double rt;

    rt = 0.;
    for(i=0;rt+3600.<t1;rt+=3600.) i++;
    *trh = i;

    for(i=0;rt+60.<t1;rt+=60.) i++;
    *trm = i;

    for(i=0;rt+1.<t1;rt+=1.) i++;
    *trs = i;

    *trss = t1-rt;
}

void filter1(int N, double *dval)
{
    int i;
    double z1,z0,B,Q;
    double v1,v0;

    Q = 0.994;
    B = 2./(1.+Q);

    z1 = dval[0];
    dval[0] = 0.;

    for(i=1;i<N;i++)
    {
	z0 = dval[i];
	dval[i] = (z0-z1)/B + Q*dval[i-1];
	z1 = z0;
    }
    dval[0] = dval[1];
}

void filter2(int N, double *dval, float dt)
{
    int i,npoles;
    double a[3],b[3];
    double corner,wc,fact,sector,wedge;
    double xm,ym,den,um,vm,sum,ang;
    double *ddat;

    ddat = (double *)malloc(N*sizeof(double));
    corner = 10.;
    npoles = 2;

    wc = corner * dt;
    fact = tan(M_PI*wc);
    sector = M_PI/npoles;
    wedge  = sector / 2.;

    ang = wedge;
    xm = -fact*cos(ang);
    ym =  fact*sin(ang);
    den = (1.-xm)*(1.-xm) + ym*ym;

    um = (1.-xm*xm -ym*ym) / den;
    vm = (2. * ym ) /den;

    b[0] = 1.;
    b[1] = -2. * um;
    b[2] = um*um + vm*vm;

    sum = b[0] + b[1] + b[2];
    a[0] = sum / 4.;
    a[1] = sum / 2.;
    a[2] = sum / 4.;

    ddat[0] = 0.;
    ddat[1] = 0.;
    for(i=2;i<N;i++)
        ddat[i] = a[0]*dval[i] + a[1]*dval[i-1] + a[2]*dval[i-2] - b[1]*ddat[i-1] - b[2]*ddat[i-2];
  
    for(i=0;i<N;i++) dval[i] = ddat[i];

    free(ddat);
}

int stalta_main(MSTrace *mst,double crt)
{
    int i,samplesize,ival,trig1;
    double dt,t1;
    double tsta,tlta,thp,tk;
    double val0,sta0,lta0,r0;
    double val1,r1,sta1,lta1,*dval;
    double start_triger_check,evdur;
    int trh,trm,trs;
    double trss,z1,z0,B,Q;
    float fval;
    void  *sptr;

    BTime itime;

    ms_hptime2btime(mst->starttime,&itime);

    dt = 1./mst->samprate;

    val1  = 0.;
    r1    = 0.;
    sta1  = 0.000;
    lta1  = 0.000;

    thp   = 0.994;
    tk    = 3.;

    tsta  = 0.05;
    tlta  = 5.0;
    evdur = 60.;
    trig1 = -1;

    t1 = (double)itime.hour*60.*60.+
	    (double)itime.min*60.+
	    (double)itime.sec + (double)itime.fract/10000.;

    if ( (samplesize = ms_samplesize(mst->sampletype)) == 0 )
    {
          fprintf (stderr, "Unrecognized sample type: %c\n", mst->sampletype);
    }

    dval = (double *)malloc(mst->numsamples*sizeof(double));
    for(i=0;i<mst->numsamples;i++)
    {
	sptr = (char*)mst->datasamples + (i * samplesize);
        if( mst->sampletype == 'i' )
        {
	    ival = *(int32_t *)sptr;
	    z0 = (double)ival;
	}
        else if( mst->sampletype == 'f' )
        {
	    fval = *(int32_t *)sptr;
	    z0 = (double)fval;
	}
        else if( mst->sampletype == 'd' )
        {
	    z0 = *(double *)sptr;
	}
	dval[i] = z0;
    }

    filter1(mst->numsamples,dval);

    filter2(mst->numsamples,dval,dt);

    start_triger_check = t1 + tlta;

    val1 = 0.;
    for(i=0;i<mst->numsamples;i++)
    {
        val0 = dval[i];
//        fprintf(stdout,"%f %f\n",t1,val0);
	stalta(val0,val1,r1,sta1,lta1,thp,tk,tsta,tlta,dt,&r0,&sta0,&lta0);
//        fprintf(stdout,"%f %f\n",t1,sta0/lta0);
	
	if( trig1 == -1 ) 
	{
	   if( t1 > start_triger_check ) trig1 = 0;
	}
	else if( trig1 == 0 )
	{
	   if( sta0 > lta0*crt)
	   {
		trig1 = 1;
		ntrig += 1;
		start_triger_check = t1 + evdur;
		cnvt_time(t1,&trh,&trm,&trs,&trss);
		fprintf(stdout,"%-4s %s  Trigger %02d:%02d:%06.3lf  %6.2lf\n",
				mst->station,mst->channel,
				trh,trm,trs+trss,sta0/lta0);
	   }
	}
	else if( trig1 == 1 && t1 > start_triger_check )
	{
	   trig1 = -1;
	}
	val1 = val0;
	z1   = z0;
	r1 = r0;
	sta1 = sta0;
	lta1 = lta0;
	t1 += dt;
    }

    free(dval);

    return 0;
}
